#include <DallasTemperature.h>
#include <ESP8266WiFi.h>
#include <OneWire.h>
#include <espnow.h>

#define TEMPERATURE_PIN D5
#define SWITCH_PIN D8
#define PUBLISH_INTERVAL 600000000

uint8_t targetAddress[] = {0xEC, 0xFA, 0xBC, 0xC2, 0xEB, 0xB6};
OneWire _oneWire = OneWire(TEMPERATURE_PIN);
DallasTemperature _ds18b20 = DallasTemperature(&_oneWire);

void sleep() {
	long sleepTime = PUBLISH_INTERVAL - micros();
	ESP.deepSleep(sleepTime);
}

void OnDataSent(uint8_t *mac_addr, uint8_t sendStatus) {
	sleep();
}

bool publishTemperature() {
	_ds18b20.requestTemperatures();
	float temperature = _ds18b20.getTempCByIndex(0);
	int attemptStart = millis();
	while ((temperature == 85 || temperature == -127)) {
		if (attemptStart - millis() > 500) {
			return false;
		}
		temperature = _ds18b20.getTempCByIndex(0);
	}
	esp_now_send(targetAddress, (uint8_t*)&temperature, sizeof(temperature));
	return true;
}

void publishDefault() {
	Serial.println("Sending default");
	float defaultValue = 100.0;
	esp_now_send(targetAddress, (uint8_t *)&defaultValue, sizeof(defaultValue));
}

void setup() {
	digitalWrite(SWITCH_PIN, HIGH);
	Serial.begin(115200);
	pinMode(TEMPERATURE_PIN, INPUT_PULLUP);
	pinMode(SWITCH_PIN, OUTPUT);

	WiFi.mode(WIFI_STA);

	if (esp_now_init() != 0) {
		Serial.println("Error initialising ESP-NOW");
		return;
	}

	esp_now_set_self_role(ESP_NOW_ROLE_CONTROLLER);
	esp_now_register_send_cb(OnDataSent);
	esp_now_add_peer(targetAddress, ESP_NOW_ROLE_SLAVE, 1, NULL, 0);

	_ds18b20.begin();
	if (!publishTemperature())
		publishDefault();

	digitalWrite(SWITCH_PIN, LOW);
}

void loop() {
}